# Environment Context

**Provides a runtime environment context (e.g. dev/stage/prod) in Drupal, with pluggable detection and metadata.**

---

## 🚀 Features

- Detects the current environment at runtime using event subscribers
- Context-aware: integrates with Layout Builder, Blocks, Views, etc.
- Configurable list of known environments (e.g. "development", "staging", "production")
- Optional metadata (labels, colors, URLs, etc.) for each environment
- Cache context (`environment`) ensures correct rendering per environment
- Includes a `Condition` plugin to restrict visibility based on environment
- Extensible via events (no hardcoded detection logic)

---

## 🔧 Installation

1. Copy this module into your `modules/custom/environment_context` directory.
2. Enable it via the Drupal admin or Drush:
   ```bash
   drush en environment_context
   ```

---

## 🧠 How It Works

This module detects the current environment (e.g. `prod`, `dev`, `stage`) at **runtime** by dispatching an event. Listeners can provide detection logic (e.g., based on hostname, env variables, etc.).

The environment is made available as a **context**, which means you can use it in:

- Block visibility conditions
- Layout Builder
- Custom plugins
- Twig templates (via optional helper)

It also defines a **cache context** to ensure output is cached per-environment correctly.

---

## 🧩 Environment Condition

Use the included `EnvironmentCondition` to control visibility based on environment:

- Go to any block
- Click "Visibility conditions"
- Choose "Current environment"
- Select which environments it should apply to

---

## 🔄 Extending the Module

### Add your own detection logic:

Implement an event subscriber for `\Drupal\environment_context\Event\EnvironmentDetectionEvent`:

```php
public static function getSubscribedEvents(): array {
  return [
    EnvironmentDetectionEvent::EVENT_NAME => 'onDetectEnvironment',
  ];
}

public function onDetectEnvironment(EnvironmentDetectionEvent $event): void {
  $env = getenv('DRUPAL_ENV') ?: 'production';
  $event->setEnvironment($env);
}
```

### Register custom environments:

Implement a subscriber for `AvailableEnvironmentsEvent`:

```php
$event->addEnvironment('ci', [
  'label' => 'CI Environment',
  'color' => '#00aaff',
  'url' => 'https://ci.example.com',
]);
```

---

## 🧪 Accessing Environment in Custom Code

```php
use Drupal\environment_context\ContextProvider\EnvironmentResolverInterface;

public function __construct(EnvironmentResolverInterface $resolver) {
  $this->resolver = $resolver;
}

$env = $this->resolver->getCurrentEnvironment();
```

To get labels and metadata:

```php
use Drupal\environment_context\Environment\EnvironmentRegistryInterface;

$definition = $registry->getDefinition('staging');
$label = $definition['label'];
```

---

## 🧰 Cache Context

This module defines the `environment` cache context.

To use it in render arrays or conditions:

```php
'#cache' => [
  'contexts' => ['environment'],
],
```

---

## 🤝 Credits

- Designed by Pascal Crott at erdfisch
- Inspired by practical deployment challenges with multi-environment Drupal setups

---

## 📝 License

This module is open source and released under the [GPL-2.0-or-later](https://www.drupal.org/licensing/faq) license.
