<?php

declare(strict_types=1);

namespace Drupal\environment_context\Event;

use Symfony\Contracts\EventDispatcher\Event;

/**
 * Event used to collect a list of available environments.
 *
 * Other modules can listen to this event and add their own environments
 * to the list using the addEnvironment() method.
 *
 * @see \Drupal\environment_context\Environment\EnvironmentRegistryInterface::getAvailableEnvironments()
 */
class AvailableEnvironmentsEvent extends Event {

  /**
   * The name of the event.
   *
   * @var string
   */
  public const EVENT_NAME = 'environment_context.available_environments';

  /**
   * The list of available environment machine names.
   *
   * @var string[]
   */
  protected array $environments = [];

  /**
   * Constructs a new AvailableEnvironmentsEvent.
   *
   * @param string[] $defaults
   *   An array of default environments to seed the event with.
   */
  public function __construct(array $defaults = []) {
    $this->environments = $defaults;
  }

  /**
   * Adds or overrides an environment definition.
   *
   * @param string $machine_name
   *   The machine name of the environment (e.g., 'production').
   * @param string|array|null $definition
   *   A human-readable label, or a full associative array of metadata.
   */
  public function addEnvironment(string $machine_name, string|array|null $definition = NULL): void {
    if (is_string($definition)) {
      $definition = ['label' => $definition];
    }
    elseif (!is_array($definition)) {
      $definition = ['label' => $machine_name];
    }

    $this->environments[$machine_name] = $definition + ['label' => $machine_name];
  }

  /**
   * Returns an array of environment definitions.
   *
   * @return array[]
   *   Array of machine names and metadata (label).
   */
  public function getEnvironments(): array {
    return $this->environments;
  }

  /**
   * Returns just the machine names.
   *
   * @return string[]
   *   Array of machine names.
   */
  public function getEnvironmentKeys(): array {
    return array_keys($this->environments);
  }

  /**
   * Sets the list of environments explicitly.
   *
   * @param string[] $environments
   *   An array of environment names.
   */
  public function setEnvironments(array $environments): void {
    $this->environments = $environments;
  }

}
