<?php

namespace Drupal\environment_context\Plugin\Condition;

use Drupal\Core\Condition\Attribute\Condition;
use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\environment_context\Environment\EnvironmentRegistryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a condition to check the current environment.
 *
 * @Condition(
 *   id = "environment",
 *   label = @Translation("Current environment"),
 *   context_definitions = {
 *     "environment" = @ContextDefinition("environment", label = @Translation("Current environment"))
 *   }
 * )
 */
#[Condition(
  id: "environment",
  label: new TranslatableMarkup("Current environment"),
  context_definitions: [
    "environment" => new ContextDefinition(
      data_type: "environment",
      label: new TranslatableMarkup("Current environment"),
    ),
  ]
)]
class EnvironmentCondition extends ConditionPluginBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a new EnvironmentCondition.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param string $plugin_id
   *   Plugin ID.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param \Drupal\environment_context\Environment\EnvironmentRegistryInterface $registry
   *   The environment registry.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected readonly EnvironmentRegistryInterface $registry
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('environment_context.registry')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return array_merge([
      'environments' => [],
    ], parent::defaultConfiguration());
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $environments = $this->registry->getAvailableEnvironments();
    if (!empty($environments)) {
      $options = [];
      foreach ($this->registry->getEnvironmentDefinitions() as $machine => $info) {
        $options[$machine] = $info['label'] ?? $machine;
      }

      $form['environments'] = [
        '#type' => 'checkboxes',
        '#title' => $this->t('Environments'),
        '#options' => $options,
        '#default_value' => $this->configuration['environments'] ?? [],
      ];
    }
    else {
      $form['environments'] = [
        '#markup' => $this->t('There are no environments available.'),
      ];
      return $form;
    }

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['environments'] = array_filter($form_state->getValue('environments') ?? []);
  }

  /**
   * {@inheritdoc}
   */
  public function evaluate(): bool {
    if ($this->isNegated()) {
      return !$this->evaluateCondition();
    }
    return $this->evaluateCondition();
  }

  /**
   * Evaluates the environment match.
   *
   * @return bool
   *   TRUE if the current environment is in the configured list.
   */
  protected function evaluateCondition(): bool {
    if (empty($this->configuration['environments'])) {
      // No environments selected; treat as "always FALSE".
      return FALSE;
    }

    $context = $this->getContextValue('environment');
    return in_array($context, $this->configuration['environments'], TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function summary(): string {
    $environments = $this->configuration['environments'] ?? [];
    if (empty($environments)) {
      return $this->t('No environments selected.');
    }

    $defs = $this->registry->getEnvironmentDefinitions();
    $labels = [];
    foreach ($environments as $env) {
      $labels[] = $defs[$env]['label'] ?? $env;
    }

    return $this->t('Environment is: @envs', ['@envs' => implode(', ', $labels)]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts(): array {
    return ['environment'];
  }

}
