<?php

declare(strict_types=1);

namespace Drupal\Tests\environment_context\Functional;

use Drupal\environment_context\Event\AvailableEnvironmentsEvent;
use Drupal\environment_context\Event\EnvironmentDetectionEvent;
use Drupal\Tests\BrowserTestBase;

/**
 * Functional test for the EnvironmentCondition plugin.
 *
 * @group environment_context
 */
class EnvironmentConditionFunctionalTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'environment_context',
    'system',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected $profile = 'standard';

  /**
   * Tests the EnvironmentCondition in a fully bootstrapped environment.
   */
  public function testConditionEvaluatesFromProviderContext(): void {
    // Provide environment via event.
    $this->container->get('event_dispatcher')->addListener(
      EnvironmentDetectionEvent::EVENT_NAME,
      static function (EnvironmentDetectionEvent $event) {
        $event->setEnvironment('production');
      }
    );

    /** @var \Drupal\environment_context\ContextProvider\EnvironmentContext $provider */
    $provider = $this->container->get('environment_context.context.environment');
    $context = $provider->getRuntimeContexts([])['environment'];

    $condition = $this->container->get('plugin.manager.condition')->createInstance('environment', [
      'environments' => ['production'],
    ]);
    $condition->setContext('environment', $context);

    $this->assertTrue($condition->evaluate(), 'Condition evaluates TRUE for production environment.');

    $condition->setConfiguration(['environments' => ['staging']]);
    $this->assertFalse($condition->evaluate(), 'Condition evaluates FALSE for non-matching environment.');
  }

  /**
   * Tests the summary text for configured environments.
   */
  public function testConditionSummary(): void {
    // Register environment definitions.
    $this->container->get('event_dispatcher')->addListener(
      AvailableEnvironmentsEvent::EVENT_NAME,
      static function (AvailableEnvironmentsEvent $event) {
        $event->addEnvironment('production', ['label' => 'Production']);
        $event->addEnvironment('staging', ['label' => 'Staging']);
      }
    );

    // Create and inject the runtime context.
    /** @var \Drupal\environment_context\ContextProvider\EnvironmentContext $provider */
    $provider = $this->container->get('environment_context.context.environment');
    $context = $provider->getRuntimeContexts([])['environment'];

    $condition = $this->container->get('plugin.manager.condition')->createInstance('environment', [
      'environments' => ['production', 'staging'],
    ]);
    $condition->setContext('environment', $context);

    $summary = $condition->summary();
    $this->assertStringContainsString('Production', $summary);
    $this->assertStringContainsString('Staging', $summary);
  }

  /**
   * Tests the summary text when no environments are configured.
   */
  public function testConditionSummaryWhenEmpty(): void {
    $condition = $this->container->get('plugin.manager.condition')->createInstance('environment', [
      'environments' => [],
    ]);

    $summary = $condition->summary();
    $this->assertNotEmpty($summary);
    $this->assertSame('No environments selected.', $summary);
  }

}
